#ifndef _EVENT_H_B
#define _EVENT_H_B
/*This file contains the class definitions of the event class */
#include "link.h"
#include "node.h"
#include "constants.h"
#include "framebuffer.h"
#include "PQE.h"
#include "frame.h"

class eventlist;  /*see PQE.h*/
class frame;      /*see frame.h*/
class linkx;       /*see link.h*/

class event
{
public:
double time ;     /* time at which the event is scheduled to occur*/
node *sourcenode; /* For an event: frame handed from node to buffer,
                     sourcenode is the pointer to the node from which the 
		     frame is handed in, for other events, it is NULL */
framebuffer * sourcebuffer ; /* For an event: frame handed from buffer to
                                link, it is the pointer to the buffer from
                                which the frame is extracted to the link,
				for other cases, it is NULL */
linkx * sourcelink ;  /* For an event: frame handed from link to node,
                        sourcelink is the pointer to the link which hands 
			in the frame to the node. For other events, it is 
			NULL */
node * destnode ;   /* For an event: frame handed from link to node, 
			destnode is the node to which the frame is being 
			given to. For other events, it is NULL */
framebuffer * destbuffer ; /*For an event: frame handed from node to 
			     buffer, it is the buffer to which the frame 
			     will be handed in. For other events, it is 
				NULL */
linkx * destlink ;   /* For an event: frame handed from buffer to link, it 
			is the link to which the frame is handed in. For 
			other events, it is NULL */
frame * thisframe;  /* thisframe is the pointer to the frame for which the 
			event is for */
           
int dataseq ; // The data sequence of the frame for which the event is for
              
SourceType sourcetype;  /*it is either Node, Buffer or Link
                          For eg, for event: Frame handed from node to 
			  buffer, sourcetype is Node. */
SourceType desttype;    /* it is either Node, Buffer or Link
                          For eg, for event: Frame handed from node to 
			  buffer, desttype is Buffer */
bool timeout;           //used to true and false time outs

public:

event();  /*constructor: just sets the value of the timeout varible to 
		"false'. Doesn't set any other variable. */

void scheduleevent(double time_t, frame *eventframe,int
event_type, node *source, node *destn, framebuffer *destbuff, 
framebuffer *sourcebuff, linkx *source_link, linkx *dest_link);

/* when an event is created, scheduleevent is called to initalize the 
event object variables.
 "time-t" -> is the time of the event
 "eventframe" -> pointer to frame fro which the event is scheduled
 "event_type" -> type of event(TO event, Event:frame handed from node to 
		 buffer, etc.)
 "source" -> is the value of the sourcenode variable for that event
 "destn"  -> is the value of the destnode variable for the event
 "destbuff" -> is the value of the destbuffer variable
 "sourcebuff" -> is the value of the sourcebuffer variable
 "source_link" -> is the value of the sourcelink variable
 "dest_link" -> is the value of the destlink variable
*/

//event handlers
void TO_handler(eventlist *header, node *eventnode, PQE *eventPQE, 
double current_time, linkx *slink);
/* TO_handler handles the TO event. 
   "header" -> pointer to the eventlist(There is only one eventlist)
   "eventnode" -> pointer to the node at which the TO event occurred
   "eventPQE" -> Pointer to the PQE node that corresponds to this event
   "current_time" -> is the time at which the event occurred.
   "slink" -> is the pointer to the link to which the buffer in the source 
	      node gives frames to
*/

void Node2Buff_handler(eventlist *header, framebuffer *ptr, double 
current_time);
/* Node2Buff_handler handles the event:Frame handed from node to buffer
   "header" -> pointer to the eventlist(There is only one list)
   "ptr" -> pointer to the framebuffer to which the frame is handed to
   "current_time" -> time at which the event occurred.
*/

void Buffer2Link_handler(eventlist *header, linkx *nextlink, double 
current_time);
/* Buffer2Link_handler handles the event:Frame handed from buffer to link
   "header" -> pointer to the eventlist(There is only one list)
   "nextlink" -> pointer to the link to which the frame is handed 
		 to
   "current_time" -> time at which the event occurred.
*/

void Link2Node_handler(node *source, node *destn, eventlist *header, 
double current_time, linkx *slink);
/* Link2Node_handler handles the event:Frame handed from link to node
   "source" -> pointer to the source node(which sends data frames)
   "destn" -> pointer to the destination node(which sends ACK/NACK)
   "header" -> pointer to the eventlist(There is only one list)
   "current_time" -> time at which the event occurred.
   "slink" -> is the pointer to which the buffer in the source node gives 
              frames to
*/

};

#endif
