#ifndef _FRAMEBUFF_H_
#define _FRAMEBUFF_H_
/* The header file contains the class definitions for the frame buffer 
class.
*/

#include "struct_h.h"
#include "link.h"
#include "node.h"
#include "PQE.h"

class eventlist;  /*see PQE.h*/
class node;       /*see node.h*/
class linkx;       /*see link.h*/

class framebuffer  
{
public:
linkx * nextlink ; /* Link to which the buffer hands frames. */
int numframe ; /* number of frames in the buffer. Excl frame being 
                 pumped out, if any.  */
double nextfinished ; /* time the frame currently exiting is ``done''
                         for this buffer. If no frame exit is in progress
                         nextfinished < current_time. e.g. -1. */

int capacity ; // maximum number of frames the buffer can contains. 
                //  excludes frame ``being pumped out'', if any. 

frames * tailheader;  /*pointer to the tail of the framebuffer
                        This is where new frames entering the buffer
                        are put */
frames * frontheader; /*pointer to the head of the framebuffer
                        This is where the frame next in line for being 
                        served resides */

public:

framebuffer(linkx * receivelink, int size); //constructor
/*receivelink - is the pointer to the link to which the buffer will give 
                the frames to.
  size        - capacity of the framebuffer
*/

void include(frame *newframe,eventlist *header,class framebuffer 
*sourcebuff, double current_time);  
/* When a node gives a new frame to a buffer, the function "include" 
   attaches the new frame to the tail of the queue. "newframe" is a 
   pointer to the new frame being added. "header" is the pointer to the
   eventlist.(There is only one eventlist). "sourcebuff" is the pointer 
   to the buffer in which the frame is to be included. "current_time" is 
   the time of the event, i.e. the (simulated) time at which the inclusion 
   occurs.
*/

void extract(class framebuffer *sourcebuff, eventlist  *neweventlist, 
double current_time); 
/* When a buffer is ready to give its front frame to the next link, the 
   function "extract" extracts the frame from the buffer and gives it to 
   the link (And updates pointers etc.).
   "sourcebuff" is the pointer to the eventlist (There is only one 
   eventlist).
   "current_time" is the time at which this occurs.
*/

void destroyframes(); //destroys all the frames in the frame buffer
                      //called when stopping the simulator

};

#endif
